/*
   FILNAME:    

   PURPOSE:   

   CREATED BY:    Per Ghosh
   ENVIRONMENT:	Visual C++


INFO:
*/

#include "stdafx.h"

#include "PGStorage.h"

#include "PGStorageManager.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNAMIC(CPGStorageManager, CObject)

HRESULT CPGStorageManager::m_hResult = S_OK;

/************************************************
   *   Constructor/Destructor 
   *
 * * *
  ***
   *
*/

CPGStorageManager::CPGStorageManager()
{
}


CPGStorageManager::~CPGStorageManager()
{
   POSITION pos;

   pos = m_listToStorage.GetHeadPosition();

   while( pos )
   {
      delete m_listToStorage.GetNext( pos );
   }
   m_listToStorage.RemoveAll();
}


/************************************************
   *   Functions 
   *
 * * *
  ***
   *
*/

/************************************************
   *   Debug
   *
 * * *
  ***
   *
*/


#ifdef _DEBUG
void CPGStorageManager::AssertValid() const
{
	CObject::AssertValid();
}

void CPGStorageManager::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG


/*-------------------------------------------------------------------------------------------------
Name:	   CreateStorage()
Type:    Function
Purpose: Creates a storage

Param:   pwName = the storage name that is to be opened
         dwFlags = flags for opening a storage

Return:  BOOL, TRUE if Ok otherwise FALSE
*/
BOOL CPGStorageManager::CreateStorage( const WCHAR* pwName, DWORD dwFlags /*=STORAGE_T_RW_SE*/)
{
   CPGStorage* pStorage;
   
   pStorage = new CPGStorage( pwName );
   
   CPGStorageManager::m_hResult = ::StgCreateDocfile( pwName, dwFlags, 0, &pStorage->m_pIStorage );

   if( FAILED(CPGStorageManager::m_hResult) )
   {
      delete pStorage;
      return FALSE;                                                         
   }                                           

   m_listToStorage.AddTail( pStorage );
   m_pstorageActive = pStorage;           // Set to active storage
   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   CreateStorage()
Type:    Function
Purpose: Creates a storage
*/
BOOL CPGStorageManager::CreateStorage( LPCSTR pszName, DWORD dwFlags /*=STORAGE_T_RW_SE*/)
{
   ASSERT( strlen(pszName) < 100 );

   WCHAR pwName[100];
         
   ::MultiByteToWideChar( CP_ACP, 
                          MB_PRECOMPOSED, 
                          pszName, 
                          -1, 
                          pwName, 
                          strlen(pszName) + 1 );

   return CreateStorage( pwName, dwFlags );
}

                                
                                                      
/*-------------------------------------------------------------------------------------------------
Name:	   OpenStorage()
Type:    Function
Purpose: Open a storage

Param:   pwName = the storage name that is to be opened
         dwFlags = flags for opening a storage

Return:  BOOL, TRUE if Ok otherwise FALSE
*/
BOOL CPGStorageManager::OpenStorage( const WCHAR* pwName, DWORD dwFlags /*=STORAGE_T_RW_SE*/)
{
   CPGStorage* pStorage;
   
   pStorage = new CPGStorage( pwName );

   CPGStorageManager::m_hResult = ::StgOpenStorage( pwName, 
                                                    NULL, 
                                                    dwFlags,
                                                    NULL, 
                                                    0, 
                                                    &pStorage->m_pIStorage );
   
   if( FAILED(CPGStorageManager::m_hResult) )
   {
      delete pStorage;
      return FALSE;
   }

   m_listToStorage.AddTail( pStorage );
   m_pstorageActive = pStorage;           // Set to active storage
   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   OpenStorage()
Type:    Function
Purpose: Open a storage
*/
BOOL CPGStorageManager::OpenStorage( LPCSTR pszName, DWORD dwFlags /*=STORAGE_T_RW_SE*/)
{
   ASSERT( strlen(pszName) < 100 );

   WCHAR pwName[100];
         
   ::MultiByteToWideChar( CP_ACP, 
                          MB_PRECOMPOSED, 
                          pszName, 
                          -1, 
                          pwName, 
                          strlen(pszName) + 1 );

   return OpenStorage( pwName, dwFlags );
}

/*-------------------------------------------------------------------------------------------------
Name:	   SetActiveStorage()
Type:    Function
Purpose: Sets a new active CPGStorage

Param:   pwName = Name of the storage that are going to be activated

Return:  BOOL, TRUE if Ok otherwise FALSE
*/
BOOL CPGStorageManager::SetActiveStorage( const WCHAR* pwName )
{
   BOOL         bReturn = FALSE;    // Returnvalue

   int          iCounter = 0;       // Counter
                                                                  
   POSITION     pos;                // position in list
   CPGStorage*  pStorage;           // Pointer to CPGStorage that exist in list

   const WCHAR* pwSearchStorageName;

   pwSearchStorageName = pwName;

   // ***
   // Do not compare the full path name (if one is sent).
   // ***
   while( iCounter )
   {
      if( pwName[iCounter] == static_cast<WCHAR>('\\') )
      {
         TRACE0("Found '\\' in string. Remove it from code (ignoring anyway) !\n");
         pwSearchStorageName == &pwName[iCounter+1];
      }

      iCounter++;
   }

   // ***
   // Find the CPGStorage that match
   // ***
   pos = m_listToStorage.GetHeadPosition();
   while( pos )
   {
      pStorage = static_cast<CPGStorage*>( m_listToStorage.GetNext( pos ) );
      if( wcsstr( pwSearchStorageName, pStorage->GetName() ) )
      {
         bReturn = TRUE;
         m_pstorageActive = pStorage;
         break;
      }
   }

   return bReturn;
}

/*-------------------------------------------------------------------------------------------------
Name:	 
Type:    
Purpose: 
*/
BOOL CPGStorageManager::SetActiveStorage( LPCSTR pszName )
{
   ASSERT( strlen(pszName) < 100 );

   WCHAR pwName[100];
         
   ::MultiByteToWideChar( CP_ACP, 
                          MB_PRECOMPOSED, 
                          pszName, 
                          -1, 
                          pwName, 
                          strlen(pszName) + 1 );

   return SetActiveStorage( pwName );
}
